.floater {
    float: left;
    margin-left: 100px;
}

@media screen and ( min-width: 1320px ) {
    .floater {
        float: none;
        margin-left: auto;
    }
    .container-fluid {
        width: 1100px;
    }
}

input {
    -moz-border-radius: 3px;
    -webkit-border-radius: 3px;
    -khtml-border-radius: 3px;
    border-radius: 3px;
    background-color: white;
    padding: 3px 6px;
    border-width: 1px 0;
}

.overall {
    position: fixed;
    left: 0;
    top: 0;
    width: 100%;
    height: 100%;
    z-index: 1313;
    margin: 50px 0 50px 0;
    background-color: rgba(0,0,0,0.7);
    display: none;
}

.fig_head {
    position: relative;
    display: none;
}

.smallcirc {
    width: 10px;
    height: 10px;
    -moz-border-radius: 5px;
    -webkit-border-radius: 5px;
    -khtml-border-radius: 5px;
    border-radius: 5px;
    position: absolute;
    z-index: 100;
}

.clickable {
    cursor: pointer;
}

.btn-offig {
    margin-top: 15px;
    width: 100%;
}

@media screen and ( max-width: 1210px ) {
    .kfigure {
        width: 100%;
    }
}

.sidenav {
    height: 100%; /* 100% Full-height */
    width: 100px;
    position: fixed; /* Stay in place */
    z-index: 1; /* Stay on top */
    top: 0;
    left: 0;
    background: #111; /* For browsers that do not support gradients */
    background: -webkit-linear-gradient(#222, #3c3c3c); /* For Safari 5.1 to 6.0 */
    background: -o-linear-gradient(#222, #3c3c3c); /* For Opera 11.1 to 12.0 */
    background: -moz-linear-gradient(#222, #3c3c3c); /* For Firefox 3.6 to 15 */
    background: linear-gradient(#222, #3c3c3c); /* Standard syntax */
    overflow-x: hidden; /* Disable horizontal scroll */
    padding-top: 50px; /* Place content 50px from the top, under topnav */
}

.rotated90 {
    position: relative;
    -webkit-transform: rotate(-90deg);  /* Chrome, Safari 3.1+ */
    -moz-transform: rotate(-90deg);  /* Firefox 3.5-15 */
    -ms-transform: rotate(-90deg);  /* IE 9 */
    -o-transform: rotate(-90deg);  /* Opera 10.50-12.00 */
    transform: rotate(-90deg);  /* Firefox 16+, IE 10+, Opera 12.10+ */
}

.message {
    margin: 20px 50px 0px 150px;
    display: none;
    max-height: 75%;
    overflow-y: auto;
}
@media screen and ( max-height: 650px ) {
    .message {
        max-height: 400px;
    }
}

.hidden {
    display: none;
}
