<html lang="en">
<head>
  <meta charset="utf-8">
  <title>jQuery UI Draggable - Default functionality</title>
  <script src="bootstrap/js/jquery.js"></script>
  <script src="bootstrap/js/jquery-ui.js"></script>
  <script>
    $(document).ready(function() {
        $( "#bla" ).draggable();
    });
  </script>
</head>
<body>

<div id="bla" style="width: 600px; height: 300px; padding: 0.5em; border: 1px solid black;">
    <p><b>Drag me around</b></p>
  <p>{{ sessions |safe }}</p>
</div>


</body>
</html>
