# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-07-25 09:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='KfitsSession',
            fields=[
                ('sid', models.CharField(max_length=32, primary_key=True, serialize=False)),
                ('timestamp', models.DateTimeField(auto_now=True)),
                ('saved_output', models.TextField(null=True)),
            ],
        ),
    ]
